@@ -235,8 +235,14 @@ bool ConfIndexW::setupTopPanel(int idx)
 {
     m_w->addParam(idx, ConfTabsW::CFPT_DNL, "topdirs", 
                   tr("Start folders"),
-                  tr("The list of folders/directories to be indexed. "
-                     "Sub-folders will be recursively processed. Default: your home."));
+                  tr("The list of folders/directories to be indexed, recursively with their "
+                     "sub-folders.<br>"
+#ifdef _WIN32
+                     "%USERPROFILE% or the '~' character expand to your user profile, which is the default "
+#else
+                     "The '~' character expands to your home directory, which is the default "
+#endif
+                     "initial value."));
 
     ConfParamW *cparam = m_w->addParam(
         idx, ConfTabsW::CFPT_DNL, "skippedPaths", tr("Skipped paths"),
@@ -317,8 +323,8 @@ bool ConfIndexW::setupTopPanel(int idx)
     m_w->enableLink(bparam, cparam, true);
 
     m_w->addParam(
-        idx, ConfTabsW::CFPT_FN, "dbdir", tr("Database directory name"),
-        tr("The name for a directory where to store the index<br>"
+        idx, ConfTabsW::CFPT_FN, "dbdir", tr("Database directory"),
+        tr("The name or path for a directory where to store the index<br>"
            "A non-absolute path is taken relative to the "
            "configuration directory. The default is 'xapiandb'."), true);
     m_w->addParam(idx, ConfTabsW::CFPT_STR, "unac_except_trans",
