@@ -18,10 +18,13 @@
 
 #include <cstdlib>
 #include <list>
-#include <iostream>
 #include <memory>
 
-#include <qapplication.h>
+#include <QApplication>
+#ifdef QAPPLICATION_CLASS
+// QAPPLICATION_CLASS is defined by singleapplication.pri
+#include "singleapplication/singleapplication.h"
+#endif
 #include <qtranslator.h>
 #include <qtimer.h>
 #include <qthread.h>
@@ -34,6 +37,7 @@
 #include <QNetworkProxy>
 
 #include "rcldb.h"
+#include "rclutil.h"
 #include "rclconfig.h"
 #include "pathut.h"
 #include "recoll.h"
@@ -44,7 +48,6 @@
 #include "ssearch_w.h"
 #include "guiutils.h"
 #include "smallut.h"
-#include "readfile.h"
 #include "uncomp.h"
 #include "cstr.h"
 #include "dynconf.h"
@@ -56,6 +59,7 @@ using std::vector;
 
 extern RclConfig *theconfig;
 AdvSearchHist *g_advshistory;
+bool semantic_enabled{false};
 
 std::mutex thetempfileslock;
 // Use a list not a vector so that contained objects have stable
@@ -173,24 +177,31 @@ static void recollCleanup()
     LOGDEB2("recollCleanup: done\n" );
 }
 
-void applyStyleSheet(const QString& qssfn)
-{
-    auto comfn = path_cat(path_cat(theconfig->getDatadir(), "examples"), "recoll-common.qss");
-    std::string qss;
-    file_to_string(comfn, qss);
-    if (!qssfn.isEmpty()) {
-        LOGDEB0("Using custom style sheet: [" << qs2path(qssfn) << "]\n");
-        string customqss;
-        file_to_string(qs2path(qssfn), customqss);
-        qss += customqss;
+#ifdef QAPPLICATION_CLASS
+#ifdef Q_OS_WINDOWS
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#endif
+
+void raiseWidget(QWidget* widget) {
+#ifdef Q_OS_WINDOWS
+    HWND hwnd = (HWND)widget->winId();
+    // check if widget is minimized to Windows task bar
+    if (::IsIconic(hwnd)) {
+        ::ShowWindow(hwnd, SW_RESTORE);
     }
-    qss = prefs.scaleFonts(qss, prefs.wholeuiscale);
-    qApp->setStyleSheet(u8s2qs(qss));
+    ::SetForegroundWindow(hwnd);
+#else
+    widget->show();
+    widget->raise();
+    widget->activateWindow();
+#endif
 }
+#endif // QAPPLICATION_CLASS
 
 extern void qInitImages_recoll();
 
-static const char *thisprog;
+static string thisprog;
 
 // BEWARE COMPATIBILITY WITH recollq OPTIONS letters
 static int    op_flags;
@@ -227,7 +238,7 @@ static const char usage [] =
     "       be last on the command line if this is used.\n"
     "       Use -t -h to see the additional non-gui options\n"
     "  -w : open minimized\n"
-    "  -W : do not open at all, only system tray icon (needs system tray!)\n"
+    "  -W : do not open at all, only create a system tray icon (needs a system tray!)\n"
     "recoll -v : print version\n"
     "recoll <url>\n"
     "   This is used to open a recoll url (including an ipath), and called\n"
@@ -238,50 +249,83 @@ Usage(void)
 {
     FILE *fp = (op_flags & OPT_h) ? stdout : stderr;
     fprintf(fp, "%s\n", Rcl::version_string().c_str());
-    fprintf(fp, "%s: Usage: %s", thisprog, usage);
+    fprintf(fp, "%s: Usage: %s", thisprog.c_str(), usage);
     exit((op_flags & OPT_h)==0);
 }
 
+// Extract the recoll_confdir from the args or env. Done early to setup the singleapp before the
+// full args processing, because we need it before creating the qapplication.
+static std::string get_config(int argc, char **argv)
+{
+    std::string aconf;
+    for (int i = 0; i < argc; i++) {
+        if (!strcmp(argv[i], "-c")) {
+            if (i < argc-1) {
+                aconf = argv[++i];
+                break;
+            }
+        }
+    }
+    if (!aconf.empty())
+        aconf = path_canon(aconf);
+    std::string econf;
+    auto cp = getenv("RECOLL_CONFDIR");
+    if (cp) {
+        econf = path_canon(cp);
+    }
+    return aconf.empty() ? econf : aconf;
+}
+
 int main(int argc, char **argv)
 {
+    pathut_setargv0(argv[0]);
+    
     // if we are named recollq or option "-t" is present at all, we
     // don't do the GUI thing and pass the whole to recollq for
     // command line / pipe usage.
-    if (!strcmp(argv[0], "recollq"))
+    if (path_getsimple(argv[0]) == "recollq")
         exit(recollq(&theconfig, argc, argv));
-    for (int i = 0; i < argc; i++) {
+    for (int i = 1; i < argc; i++) {
         if (!strcmp(argv[i], "-t")) {
             exit(recollq(&theconfig, argc, argv));
         }
     }
 
-#ifdef USING_WEBENGINE_nono
-    // Note: this is not used any more because we use a file:// baseUrl so that local access is
-    // allowed (and network access should not, but it fact it is, so we forbid it by setting a
-    // bogus proxy).
+    QCoreApplication::setOrganizationName("Recoll.org");
+    QCoreApplication::setApplicationName("recoll");
 
-    // Old comment, kept around in case we change baseurl again:
-    // The following is necessary for allowing webengine to load local resources (icons)
-    // It is not an issue because we never access remote sites.
-    char arg_disable_web_security[] = "--disable-web-security";
-    int appargc = argc + 1;
-    char** appargv = new char*[appargc+1];
-    for(int i = 0; i < argc; i++) {
-        appargv[i] = argv[i];
+    // Do this very early because of the singleapplication choice. We'll have to do it again once we
+    // have a db config
+    rwSettings(false);
+
+#ifdef QAPPLICATION_CLASS
+    // If the pref for single app is not set, then allow further instances.
+    bool allowsecund = !prefs.singleapp;
+#ifdef _WIN32
+    // On Windows, allow further instances even if prefs.singleapp is set, but then the
+    // secundary will just ping the primary and exit.
+    allowsecund = true;
+#endif // _WIN32
+    auto confforsa = get_config(argc, argv);
+    SingleApplication app(argc, argv, allowsecund, SingleApplication::User, 1000, u8s2qs(confforsa));
+#ifdef _WIN32
+    if (prefs.singleapp && app.isSecondary()) {
+        AllowSetForegroundWindow(DWORD(app.primaryPid()));
+        app.sendMessage("RAISE_WIDGET");
+        return 0;
     }
-    appargv[argc] = arg_disable_web_security;
-    appargv[argc+1] = nullptr;
-    QApplication app(appargc, appargv);
+#endif // _WIN32
 #else
     QApplication app(argc, argv);
 #endif
 
 #if defined(USING_WEBENGINE) || defined(USING_WEBKIT)
-    // Prevent WebEngine HTTP connections by setting a bogus proxy. This is for preventing network
-    // accesses from webengine when it is used to implement the preview window: depending on the
-    // loaded document, these could be unsafe. Otoh this also means that the display will be less
-    // nice than it could be if we let the engine load necessary javascript etc.  Note that this may
-    // still be less safe than using a QTextBrowser (or webkit probably) for the Preview.
+    // Prevent Webkit HTTP connections by setting a bogus proxy. This is for preventing network
+    // accesses from webkit when it is used to implement the preview window. E.g. this is what
+    // blocks fetching images from external sites.
+    // This only partially works for for webengine (ok for image fetches, not JS). We use a request
+    // interceptor for blocking external fetches, see rclwebpage.h.
+    // Note that this may still be less safe than using a QTextBrowser for the Preview.
     QNetworkProxy proxy;
     proxy.setType(QNetworkProxy::Socks5Proxy);
     proxy.setHostName("127.0.0.1");
@@ -290,45 +334,38 @@ int main(int argc, char **argv)
     QNetworkProxy::setApplicationProxy(proxy);
 #endif // Real browser engines
     
-    QCoreApplication::setOrganizationName("Recoll.org");
-    QCoreApplication::setApplicationName("recoll");
-
     string a_config;
     string a_lang;
     string question;
     string urltoview;
 
-    // Avoid disturbing argc and argv. Especially, setting argc to 0
-    // prevents WM_CLASS to be set from argv[0] (it appears that qt
-    // keeps a ref to argc, and that it is used at exec() time to set
-    // WM_CLASS from argv[0]). Curiously, it seems that the argv
-    // pointer can be modified without consequences, but we use a copy
-    // to play it safe
-    int myargc = argc;
-    char **myargv = argv;
-    thisprog = myargv[0];
-    myargc--; myargv++;
-    
-    while (myargc > 0 && **myargv == '-') {
-        (*myargv)++;
-        if (!(**myargv))
+    QStringList arguments = app.arguments();
+    thisprog = arguments.takeAt(0).toStdString();
+
+    while (arguments.size() > 0 && arguments.at(0).startsWith('-')) {
+        if (arguments.at(0).size() < 2)
             Usage();
-        while (**myargv)
-            switch (*(*myargv)++) {
+        int argidx = 0;
+        int maxidx = arguments.at(0).size();
+        while (++argidx < maxidx) {
+            switch (arguments.at(0).at(argidx).toLatin1()) {
             case 'a': op_flags |= OPT_a; break;
-            case 'c':   op_flags |= OPT_c; if (myargc < 2)  Usage();
-                a_config = *(++myargv);
-                myargc--; goto b1;
+            case 'c':   op_flags |= OPT_c; if (arguments.size() < 2)  Usage();
+                a_config = arguments.takeAt(1).toStdString();
+                maxidx = 0;  // end inner loop
+                break;
             case 'f': op_flags |= OPT_f; break;
-            case 'h': op_flags |= OPT_h; Usage();break;
-            case 'L':   op_flags |= OPT_L; if (myargc < 2)  Usage();
-                a_lang = *(++myargv);
-                myargc--; goto b1;
+            case 'h': op_flags |= OPT_h; Usage(); break;
+            case 'L':   op_flags |= OPT_L; if (arguments.size() < 2)  Usage();
+                a_lang = arguments.takeAt(1).toStdString();
+                maxidx = 0;  // end inner loop
+                break;
             case 'l': op_flags |= OPT_l; break;
             case 'o': op_flags |= OPT_o; break;
-            case 'q':   op_flags |= OPT_q; if (myargc < 2)  Usage();
-                question = *(++myargv);
-                myargc--; goto b1;
+            case 'q':   op_flags |= OPT_q; if (arguments.size() < 2)  Usage();
+                question = arguments.takeAt(1).toStdString();
+                maxidx = 0;  // end inner loop
+                break;
             case 't': op_flags |= OPT_t; break;
             case 'v': op_flags |= OPT_v;
                 fprintf(stdout, "%s\n", Rcl::version_string().c_str());
@@ -337,26 +374,25 @@ int main(int argc, char **argv)
             case 'w': op_flags |= OPT_w; break;
             default: Usage();
             }
-    b1: myargc--; myargv++;
+        }
+        arguments.removeAt(0);
     }
 
     // If -q was given, all remaining non-option args are concatenated
     // to the query. This is for the common case recoll -q x y z to
     // avoid needing quoting "x y z"
-    if (op_flags & OPT_q)
-        while (myargc > 0) {
-            question += " ";
-            question += *myargv++;
-            myargc--;
-        }
+    if (op_flags & OPT_q && arguments.size() > 0) {
+        question += ' ' + arguments.join(' ').toStdString();
+        arguments.clear();
+    }
 
     // Else the remaining argument should be an URL to be opened
-    if (myargc == 1) {
-        urltoview = *myargv++;myargc--;
+    if (arguments.size() == 1) {
+        urltoview = arguments.takeAt(0).toStdString();
         if (urltoview.compare(0, 7, cstr_fileu)) {
             Usage();
         }
-    } else if (myargc > 0)
+    } else if (arguments.size() > 0)
         Usage();
 
 
@@ -370,13 +406,17 @@ int main(int argc, char **argv)
     }
     //    fprintf(stderr, "recollinit done\n");
 
-    // Translations for Qt standard widgets
+    // Interface language
     QString slang;
     if (op_flags & OPT_L) {
         slang = u8s2qs(a_lang);
+    } else if (!prefs.uilanguage.isEmpty()) {
+        slang = prefs.uilanguage;
     } else {
-        slang = QLocale::system().name().left(2);
+        slang = QLocale::system().name();
     }
+
+    // Translations for Qt standard widgets
     QTranslator qt_trans(0);
     if (qt_trans.load(QString("qt_%1").arg(slang), 
 #if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
@@ -391,12 +431,19 @@ int main(int argc, char **argv)
     // Translations for Recoll
     string translatdir = path_cat(theconfig->getDatadir(), "translations");
     QTranslator translator(0);
-    auto loaded = translator.load(QString("recoll_") + slang,
-                                  translatdir.c_str());
+    auto loaded = translator.load(QString("recoll_") + slang, translatdir.c_str());
     if (loaded)
         app.installTranslator(&translator);
 
-    //    fprintf(stderr, "Translations installed\n");
+    
+#ifdef ENABLE_SEMANTIC
+    {
+        std::string unused;
+        if (theconfig->getConfParam("sem_venv", unused) && !unused.empty()) {
+            semantic_enabled = true;
+        }
+    }
+#endif // SEMANTIC        
 
     string historyfile = path_cat(theconfig->getConfDir(), "history");
     g_dynconf = new RclDynConf(historyfile);
@@ -412,7 +459,7 @@ int main(int argc, char **argv)
     rwSettings(false);
     //    fprintf(stderr, "Settings done\n");
 
-    applyStyleSheet(prefs.qssFile);
+    applyStyle();
     
     QIcon icon;
     icon.addFile(QString::fromUtf8(":/images/recoll.png"));
@@ -422,6 +469,14 @@ int main(int argc, char **argv)
     RclMain w;
     mainWindow = &w;
 
+#ifdef QAPPLICATION_CLASS
+#ifdef Q_OS_WINDOWS
+    QObject::connect(&app, &SingleApplication::receivedMessage, &w, [&w] () { raiseWidget(&w); } );
+#else
+    QObject::connect(&app, &SingleApplication::instanceStarted, &w, [&w] () { raiseWidget(&w); } );
+#endif
+#endif
+
     string dbdir = theconfig->getDbDir();
     if (dbdir.empty()) {
         QMessageBox::critical(
@@ -450,8 +505,6 @@ int main(int argc, char **argv)
     }
     QTimer::singleShot(0, mainWindow, SLOT(initDbOpen()));
 
-    mainWindow->sSearch->searchTypCMB->setCurrentIndex(prefs.ssearchTyp);
-    mainWindow->sSearch->onSearchTypeChanged(prefs.ssearchTyp);
     if (op_flags & OPT_q) {
         SSearch::SSearchType stype;
         if (op_flags & OPT_o) {
@@ -463,7 +516,7 @@ int main(int argc, char **argv)
         } else {
             stype = SSearch::SST_LANG;
         }
-        mainWindow->sSearch->searchTypCMB->setCurrentIndex(int(stype));
+        mainWindow->sSearch->onSearchTypeChanged(stype);
         mainWindow->sSearch->setSearchString(QString::fromLocal8Bit(question.c_str()));
     } else if (!urltoview.empty()) {
         LOGDEB("MAIN: got urltoview [" << urltoview << "]\n");
