@@ -53,6 +53,7 @@ class ActSearchW;
 class IdxTreeModel;
 class UIPrefsDialog;
 class ConfigSwitchW;
+class PTransEdit;
 
 #include "ui_rclmain.h"
 
@@ -121,12 +122,17 @@ public slots:
     virtual void showMissingHelpers();
     virtual void showActiveTypes();
     virtual void startOnlineManual();
+#ifdef _WIN32
+    virtual void checkWinVersion();
+#endif 
     virtual void startManual(const std::string& = std::string());
     virtual void showDocHistory();
     virtual void showExtIdxDialog();
     virtual void setSynEnabled(bool);
     virtual void showUIPrefs();
     virtual void showIndexConfig();
+    virtual void showPTrans(const QString& = "");
+    virtual void delPTrans();
     virtual void execIndexConfig();
     virtual void showConfigSwitch();
     virtual void showCronTool();
@@ -148,7 +154,7 @@ public slots:
                                    int line = -1, bool enterHistory=true);
     virtual void openWith(Rcl::Doc, std::string);
     virtual void saveDocToFile(Rcl::Doc);
-    virtual void populateSideFilters(SideFilterUpdateReason);
+    virtual void populateSideFilters(RclMain::SideFilterUpdateReason);
     virtual void previewNextInTab(Preview *, int sid, int docnum);
     virtual void previewPrevInTab(Preview *, int sid, int docnum);
     virtual void previewExposed(Preview *, int sid, int docnum);
@@ -165,9 +171,9 @@ public slots:
     virtual void onFragmentsChanged();
     virtual void initDbOpen();
     virtual void toggleFullScreen();
-    virtual void on_actionSortByDateAsc_toggled(bool on);
-    virtual void on_actionSortByDateDesc_toggled(bool on);
-    virtual void on_actionShowResultsAsTable_toggled(bool on);
+    virtual void sortByDateAsc(bool on);
+    virtual void sortByDateDesc(bool on);
+    virtual void showResultsAsTable(bool on);
     virtual void onSortDataChanged(DocSeqSortSpec);
     virtual void onExtSortDataChanged(DocSeqSortSpec);
     virtual void resultCount(int);
@@ -210,6 +216,7 @@ private:
     AdvSearch      *asearchform{0};
     UIPrefsDialog  *uiprefs{0};
     ConfIndexW     *indexConfig{0};
+    PTransEdit     *pathTrans{0};
     IdxSchedW      *indexSched{0};
 #ifdef _WIN32
     WinSchedToolW  *cronTool{0};
@@ -269,7 +276,7 @@ private:
     // preview (if no ext app set)
     QString          m_urltoview;
     RclTrayIcon     *m_trayicon{0};
-   // We sometimes take the indexer lock (e.g.: when editing the webcache)
+    // We sometimes take the indexer lock (e.g.: when editing the webcache)
     Pidfile         *m_pidfile{0};
     IdxTreeModel    *m_idxtreemodel{nullptr};
     
