@@ -42,6 +42,7 @@
 #include <QKeyEvent>
 #include <QClipboard>
 #include <QToolTip>
+#include <QProgressDialog>
 
 #include "recoll.h"
 #include "docseq.h"
@@ -446,7 +447,14 @@ void RecollModel::saveAsCSV(std::fstream& fp)
     if (!m_source)
         return;
     m_source->setqquantum(30000);
-    
+
+    // The row count at this stage is generally largely underestimated. The proportion varies from 1
+    // to 4, and is unpredictable. We'd have to actually fetch the results, but this takes time.
+    int totRowsForProgress  = rowCount() * 1.5;
+    QProgressDialog progress(tr("Saving to CSV"), tr("Cancel"), 0, totRowsForProgress);
+    if (totRowsForProgress > 5000) {
+        progress.show();
+    }
     int cols = columnCount();
     vector<string> tokens;
 
@@ -467,6 +475,12 @@ void RecollModel::saveAsCSV(std::fstream& fp)
         }
         csv = stringsToCSV(tokens);
         fp << csv << "\n";
+        if (row % 1000 == 0) {
+            qApp->processEvents();
+            if (progress.wasCanceled())
+                return;
+            progress.setValue(row);
+        }
     }
     m_source->setqquantum(100);
 }
@@ -653,7 +667,8 @@ void ResTable::init(QStringList _ifields)
     
     connect(tableView, SIGNAL(customContextMenuRequested(const QPoint&)),
             this, SLOT(createPopupMenu(const QPoint&)));
-
+    connect(tableView, SIGNAL(entered(const QModelIndex&)),
+            this, SLOT(tableView_entered(const QModelIndex&)));
     QHeaderView *header = tableView->horizontalHeader();
     if (header) {
         if (_ifields.empty()) {
@@ -1007,10 +1022,10 @@ void ResTable::onTableView_currentChanged(const QModelIndex& index)
     if (!displaydone && showmeta) {
         m_pager->displaySingleDoc(theconfig, m_detaildocnum, m_detaildoc, m_model->m_hdata);
     }
-    emit(detailDocChanged(doc, m_model->getDocSource()));
+    emit detailDocChanged(doc, m_model->getDocSource());
 }
 
-void ResTable::on_tableView_entered(const QModelIndex& index)
+void ResTable::tableView_entered(const QModelIndex& index)
 {
     LOGDEB2("ResTable::on_tableView_entered(" << index.row() << ", "  <<
             index.column() << ")\n");
@@ -1139,7 +1154,7 @@ void ResTable::onLinkClicked(const QUrl &qurl)
     switch (what) {
         // Open abstract/snippets window
     case 'A':
-        emit(showSnippets(m_detaildoc));
+        emit showSnippets(m_detaildoc);
         break;
     case 'D':
     {
